#include <ESPMegaPRO_OOP.hpp>
ESPMegaPRO::ESPMegaPRO() {
    
}
void ESPMegaPRO::begin() {
    Wire.begin(14, 33);
    inputs.begin();
    outputs.begin();
    fram.begin(FRAM_ADDRESS);
    uint8_t pinMap[16] = {0, 1, 2, 3, 4, 5, 6, 7, 15, 14, 13, 12, 11, 10, 9, 8};
    inputs.loadPinMap(pinMap);
}
void ESPMegaPRO::loop() {
    inputs.loop();
    outputs.loop();
    for (int i = 0; i < 256; i++) {
        if (cardInstalled[i]) {
            cards[i]->loop();
        }
    }   
}
void ESPMegaPRO::installCard(uint8_t slot, ExpansionCard* card) {
    cards[slot] = card;
    cardInstalled[slot] = true;
    cardCount++;
    card->begin();
}
bool ESPMegaPRO::updateTimeFromNTP() {
    struct tm timeinfo;
    if (getLocalTime(&timeinfo))
    {
        rtctime_t rtctime = this->getTime();
        if (rtctime.hours != timeinfo.tm_hour || rtctime.minutes != timeinfo.tm_min ||
            rtctime.seconds != timeinfo.tm_sec || rtctime.day != timeinfo.tm_mday ||
            rtctime.month != timeinfo.tm_mon + 1 || rtctime.year != timeinfo.tm_year + 1900)
        {
            this->setTime(timeinfo.tm_hour, timeinfo.tm_min, timeinfo.tm_sec,
                            timeinfo.tm_mday, timeinfo.tm_mon + 1, timeinfo.tm_year + 1900);
            
        }
        return true;
    }
    return false;
}
rtctime_t ESPMegaPRO::getTime() {
    tmElements_t timeElement;
    RTC.read(timeElement);
    rtctime_t time;
    time.hours = timeElement.Hour;
    time.minutes = timeElement.Minute;
    time.seconds = timeElement.Second;
    time.day = timeElement.Day;
    time.month = timeElement.Month;
    time.year = timeElement.Year + 1970;
    return time;
}

void ESPMegaPRO::setTime(int hours, int minutes, int seconds, int day, int month, int year)
{
    tmElements_t timeElement;
    timeElement.Hour = hours;
    timeElement.Minute = minutes;
    timeElement.Second = seconds;
    timeElement.Day = day;
    timeElement.Month = month;
    timeElement.Year = year - 1970;
    RTC.write(timeElement);
}

